/*
  GUIShell
  (c) 2002-2007 Jeffrey Bedard
  antiright@gmail.com

  This file is part of GUIShell.

  GUIShell is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  GUIShell is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with GUIShell; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor,
  Boston, MA  02110-1301  USA
*/

#include "gtkshell.h"

static void
gsh_delete_GSH (GSH * gsh)
{
  /* Delete atomic members.  */
  GSH_FREE_IF (gsh->geometry);
  /* Delete child classes.  */
  ARDELETE (gsh->rows);
  ARDELETE (gsh->terminal);
  ARDELETE (gsh->update);
  arfree(gsh);
}

static inline void
gsh_manage_generic (GSH * gsh, GtkWidget * widget)
{
  gtk_widget_show (widget);
  gsh->rows->current++;
  gsh->widgets.last_managed = widget;
}

static void
gsh_manage (GSH * gsh, GtkWidget * widget)
{
  if (G_LIKELY (!GSH_FLAG (GSH_PANE_NEXT)))
    {
      struct GSHRows *rows = gsh->rows;
      GtkWidget *container = rows->v;

      if (GTK_IS_MENU (container) || GTK_IS_MENU_BAR (container)
	  || GTK_IS_MENU_ITEM (container))
	{
	  gtk_menu_shell_append (GTK_MENU_SHELL (container), widget);
	  return;
	}
      gsh_manage_generic (gsh, widget);
      $ (rows, check);
      if (GSH_FLAG (GSH_NO_EXPAND))
	GSHCA_TIGHT (container, widget);
      else
	GSHCA (container, widget);
    }
  else
    {
      gsh_pane_previous (gsh, widget);
      gsh_manage_generic (gsh, widget);
    }
}

void
gsh_GSH (GSH * gsh)
{
  /* setup child classes */
  gsh->update=NULL;
  gsh->rows = ARNEW (gsh, GSHRows);
  gsh->terminal=NULL;
  /* setup methods */
  gsh->delete = &gsh_delete_GSH;
  gsh->parse = &gsh_parse_arguments;
  gsh->finalize = &gsh_GSH_finalize;
  gsh->manage = &gsh_manage;
  gsh->add.window = &gsh_GSH_add_window;
  gsh->add.button = &gsh_add_button;
  gsh->add.label = &gsh_add_label;
  /* setup app mode fields */
#define GWA gsh->widgets.app
  GWA.menubar=GWA.toolbar=GWA.status=GWA.frame=GWA.row=NULL;
#define GWAM GWA.menus
  GWAM.file=GWAM.edit=GWAM.view=GWAM.tools=GWAM.help=NULL;
  gsh->widgets.last_managed = NULL;
  gsh->editor.widget=NULL;
  gsh->command_dictionary = NULL;
  gsh->editor.fgcolor = gsh->editor.bgcolor = gsh->editor.filename
    = gsh->geometry = gsh->usage = (gchar *) NULL;
  gsh->bflags = 0;
  gsh->icon_size = GTK_ICON_SIZE_DND;
  /* Add initial window.  */
  $ (gsh, add.window);
}

GSH *
gsh_new_GSH ()
{
  GSH *gsh;

  gsh = (GSH *) armalloc (sizeof (GSH));
  gsh_GSH (gsh);

  return gsh;
}
