/*
  GUIShell
  (c) 2008-2010 Jeffrey Bedard
  antiright@gmail.com

  This file is part of GUIShell.

  GUIShell is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  GUIShell is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with GUIShell; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/


#include "rootcat.h"


static inline void
rootcat_end (StringRenderer * rc, int exit_code)
{
  $ (rc, delete);
  exit (exit_code);
}

static void
clear ()
{
  StringRenderer *rc;

  rc = new_StringRenderer_root (NULL, NULL);
  $ (rc, clear);
  $ (rc, delete);
}

static void
print_usage (const char *app_name, int exit_code)
{
  if (exit_code != 0)
    fprintf (stderr, "%s:  Invalid argument\n", app_name);
  printf ("USAGE %s [-CFcf] [string] ...\n", app_name);
  exit (exit_code);
}

int
main (int argc, char **argv)
{
  StringRenderer *rc = NULL;
  int counter = 0;
  int line = 0;
  char *color = NULL;
  char *font = NULL;

  while (++counter < argc)
    {
      const char *string = argv[counter];

      if (string[0] == '-')
	{
	  switch (string[1])
	    {
	    case 'C':		/* Set color */
	      counter++;
	      if (argv[counter])
		color = argv[counter];
	      else
		print_usage (argv[0], 1);
	      break;
	    case 'c':		/* Clean up previous invocations.  */
	      clear ();
	      break;
	    case 'F':		/* Set font */
	      counter++;
	      if (argv[counter])
		font = argv[counter];
	      else
		print_usage (argv[0], 1);
	      break;
	    case 'h':
	      print_usage (argv[0], 0);
	    default:
	      print_usage (argv[0], 1);
	    }
	}
      else
	{
	  if (!rc)
	    {
	      rc = new_StringRenderer_root (font, color);
	      $ (rc, clear);
	    }
	  {
	    unsigned int height = $ (rc, get_line_height, string);

	    $ (rc, draw, (height / 2),
	       ((++line) * LINE_SPACING * height), string);
	  }
	}
    }
  if (rc)
    $ (rc, delete);

  return 0;
}
